#include "kinect_tilt_controller/kinect_tilt_controller.h"
#include <pluginlib/class_list_macros.h>

namespace kinect_tilt_controller_ns {


/// Controller initialization in non-realtime
bool KinectTiltControllerClass::init(pr2_mechanism_model::RobotState *robot,
                            ros::NodeHandle &n)
{
  std::string joint_name;
  if (!n.getParam("joint_name", joint_name))
  {
    ROS_ERROR("No joint given in namespace: '%s')",
              n.getNamespace().c_str());
    return false;
  }

  joint_state_ = robot->getJointState(joint_name);
  if (!joint_state_)
  {
    ROS_ERROR("KinectTiltController could not find joint named '%s'",
              joint_name.c_str());
    return false;
  }


  srv_ = n.advertiseService("set_target_theta", 
                            &KinectTiltControllerClass::setTargetTheta, this);

  target_theta = 0.25 * M_PI;   // 45 deg. down angle, assuming frame of reference makes 0 degs. horizontal

  return true;
}

/// Service call to set desired position

bool KinectTiltControllerClass::setTargetTheta(kinect_tilt_controller::SetTargetTheta::Request& req,
				       kinect_tilt_controller::SetTargetTheta::Response& resp)
{
  // make sure target is allowable given joint limits

  if (req.target_theta > max_theta) {
    target_theta = max_theta;
    //    ROS_INFO("desired %lf clamped to %lf\n", target_theta, max_theta);
  }
  else if (req.target_theta < min_theta) {
    target_theta = min_theta;
    //    ROS_INFO("desired %lf clamped to %lf\n", target_theta, min_theta);
  }
  else
    target_theta = req.target_theta;

  resp.target_theta = target_theta;

  return true;
}

/// Controller startup in realtime
void KinectTiltControllerClass::starting()
{
  init_pos_ = joint_state_->position_;
  k_p = 0.1;
  min_theta = -0.5 * M_PI;
  max_theta = 0.5 * M_PI;
}


/// Controller update loop in realtime

void KinectTiltControllerClass::update()
{
  // where are we now?

  double current_theta = joint_state_->position_;

  // put out effort

  joint_state_->commanded_effort_ = k_p * (target_theta - current_theta);

  // report

  //  ROS_INFO("current %lf desired %lf commanded %lf\n", current_theta, target_theta, joint_state_->commanded_effort_);
}


/// Controller stopping in realtime
void KinectTiltControllerClass::stopping()
{
  joint_state_->commanded_effort_ = 0.0;
}

} // namespace


/// Register controller to pluginlib
PLUGINLIB_DECLARE_CLASS(kinect_tilt_controller,KinectTiltControllerPlugin, 
                         kinect_tilt_controller_ns::KinectTiltControllerClass, 
                         pr2_controller_interface::Controller)
